

#include <string.h>

#include "global.h"
#include "pstring.h"

#include "vdis.h"


char TOKEN_APP_STR[]      = "-dis_application";
char TOKEN_SITE_STR[]     = "-dis_site";
char TOKEN_EXERCISE_STR[] = "-dis_exercise";

char TOKEN_COCKPIT_APP_STR[]  = "-cockpit_application";
char TOKEN_COCKPIT_SITE_STR[] = "-cockpit_site";
char TOKEN_COCKPIT_ID_STR[]   = "-cockpit_id";
char TOKEN_COCKPIT_CODE_STR[] = "-cockpit_code";

char TOKEN_CENTER_LATITUDE_STR[]  = "-dis_latitude";
char TOKEN_CENTER_LONGITUDE_STR[] = "-dis_longitude";

char TOKEN_MINICREW_STR[] = "-minicrew";


/* **************************************************************
************************************************************** */
dis_iface::dis_iface() {

   exercise_id = 1;
   site_id = 69;
   app_id = 69;

   center.longitude = 0;
   center.latitude  = 0;

   id_counter = 1;
   
   cockpit_code = 0;
   
   control_platform.dis_id.site = control_platform.dis_id.application = control_platform.dis_id.entity = -1; 
}


/* **************************************************************
************************************************************** */
void dis_iface::init() {

   symbol_correlation_type *str;
    
   iface::init();

   disiface.init(0, NULL, CONFIG_TYPE_CLIENT);
   disiface.new_address(&clientudp, 0);
   set_dis_interface(exercise_id, site_id, app_id);
   init_disvissim(&center);

   init_earth(center.latitude, center.longitude, EARTH_RADIUS);

   id_counter = 1;

   if (interface_flag & INTERFACE_FLAG_MINICREW) {
      str = (symbol_correlation_type *)symbol_lut.find(cockpit_code);
   
      if (str)
         control_platform.init(this, str);
   }
      
}


/* **************************************************************
************************************************************** */
void dis_iface::cleanup() {

   disiface.delete_address(0);
   clientudp.dest();

   iface::cleanup();
}


/* **************************************************************
************************************************************** */
void dis_iface::parse_input(int argc, char **argv) {

   int i;
   
   for (i=0; i < argc; i++)
      switch (*(argv[i]+1)) {

         case 'c':
            if (!strcmp(argv[i], TOKEN_COCKPIT_SITE_STR)) {
               i++;

               if (i >= argc)
                  break;

               control_platform.dis_id.site = atoi(argv[i]);
               break;
            }

            if (!strcmp(argv[i], TOKEN_COCKPIT_APP_STR)) {
               i++;

               if (i >= argc)
                  break;

               control_platform.dis_id.application = atoi(argv[i]);
               break;
            }

            if (!strcmp(argv[i], TOKEN_COCKPIT_ID_STR)) {
               i++;

               if (i >= argc)
                  break;

               control_platform.dis_id.entity = atoi(argv[i]);
               break;
            }

            if (!strcmp(argv[i], TOKEN_COCKPIT_CODE_STR)) {
               i++;

               if (i >= argc)
                  break;

               cockpit_code = atoi(argv[i]);
               break;
            }

            break;

         case 'd':
            if (!strcmp(argv[i], TOKEN_APP_STR)) {
               i++;

               if (i >= argc)
                  break;

               app_id = atoi(argv[i]);
               break;
            }

            if (!strcmp(argv[i], TOKEN_SITE_STR)) {
               i++;

               if (i >= argc)
                  break;

               site_id = atoi(argv[i]);
               break;
            }

            if (!strcmp(argv[i], TOKEN_EXERCISE_STR)) {
               i++;

               if (i >= argc)
                  break;

               exercise_id = atoi(argv[i]);
               break;
            }

            if (!strcmp(argv[i], TOKEN_CENTER_LATITUDE_STR)) {
               i++;

               if (i >= argc)
                  break;

               center.latitude = deg2rad(atof(argv[i]));
               break;
            }

            if (!strcmp(argv[i], TOKEN_CENTER_LONGITUDE_STR)) {
               i++;

               if (i >= argc)
                  break;

               center.longitude = deg2rad(atof(argv[i]));
               break;
            }

            break;

         case 'm':
            if (!strcmp(argv[i], TOKEN_MINICREW_STR)) {
               interface_flag |= INTERFACE_FLAG_MINICREW;
               break;
	    }

            break;
    	     
         default:
            break;
      }

   clientudp.init(argc, argv, CONFIG_TYPE_CLIENT);

   iface::parse_input(argc, argv);
}


/* **************************************************************
************************************************************** */
dis_platform *dis_iface::AddPlatform(DISPDU_entity_state_type *es, double *machine_time) {

   dis_platform *dtr;
   char token[MAXSTRLEN];
   symbol_correlation_type *str;
   
   append(dtr = new dis_platform, NULL);

   sprintf(token, "%1d:%02d:%03d:%02d:%02d:%02d:%02d:00", es->entity_type.kind, es->entity_type.domain, es->entity_type.country, es->entity_type.category, es->entity_type.subcategory, es->entity_type.specific, es->entity_type.extra); 

   str = (symbol_correlation_type *)symbol_lut.find(token);

   if (!str) {
      sprintf(perror_buffer, "Warning: Unable to identify %s\n", token);
      pprintf(perror_buffer);
      return NULL;
   }

   dtr->type = str->sym_id;
   
   // Note: following 2 lines must be before dtr->init()      
   if (es->site_id == control_platform.dis_id.site && es->application_id == control_platform.dis_id.application && es->entity_id == control_platform.dis_id.entity) {
      dtr->platform_flag |= PLATFORM_FLAG_CONTROL;
      dtr->platform_flag &= ~PLATFORM_FLAG_DEAD_RECOG;
   }

   if (!dtr->init(this, es, str)) {
      sprintf(perror_buffer, "Warning... Can't find visual for %d\n", dtr->type);
      pprintf(perror_buffer);
   }

   return dtr;
}


/* **************************************************************
************************************************************** */
void dis_iface::update(double *machine_time) {

   DISPDU_entity_state_type es;
   int type;
   buffer_type buffer;
   dis_platform *dtr;
   double multiple = 1.0;
   double currtime;
   basic_platform *ptr;

   currtime = *machine_time;

   for (ptr=(basic_platform *)head; ptr; ptr=(basic_platform *)ptr->next)
      if (!(ptr->platform_flag & (PLATFORM_FLAG_CONTROL | PLATFORM_FLAG_EXTERNAL | PLATFORM_FLAG_DEAD)))
         ptr->update(this, &currtime, &multiple, machine_time, 0);

   if (disiface.receive_message(&buffer)) {
      do {
      
         type = buffer.data[2];

         switch (type) {

            case DIS_PDU_TYPE_ENTITY_STATE:
               es.extract(&buffer);

               for (dtr = (dis_platform *)head; dtr && (dtr->dis_id.site != es.site_id || dtr->dis_id.application != es.application_id || dtr->dis_id.entity != es.entity_id); dtr = (dis_platform *)dtr->next);

               if (!dtr) {
                  dtr = AddPlatform(&es, machine_time);
                  if (!dtr)
                     break;
               }

               dtr->update(this, &es, machine_time);
               break;

            default:
               break;
         }

      } while (disiface.receive_message(&buffer));

   }

}


/* **************************************************************
************************************************************** */
void dis_iface::mcp(plat_stat_type *pc) {

   if (!(interface_flag & INTERFACE_FLAG_MINICREW))
      return;

   control_platform.update_platform(pc);
}

